subroutine house
use params
use mini
implicit none
include 'mpif.h'

!variables to be collected and shared using MPI
double precision, allocatable :: cmpi(:), lmpi(:), ampi(:), vmpi(:), medicaidmpi(:), netmpi(:)
double precision, allocatable :: crecmpi(:), lrecmpi(:), arecmpi(:), vrecmpi(:), medicaidrecmpi(:), netrecmpi(:)
integer(kind=1), allocatable :: dfmpi(:), dgmpi(:), dmmpi(:)
integer(kind=1), allocatable :: dfrecmpi(:), dgrecmpi(:), dmrecmpi(:)

!variables to compute value of foodstamps
integer :: is, ih, inds(2)
double precision :: VM(nh,ns), vals(2), temp(nh)
double precision :: vnotpay
integer :: gcp

!accuracy of brent minimization
double precision, parameter :: xacc=1e-11

!for mpi
integer :: npt, itopt, iendt
integer, parameter :: nit=J*ng*ne*nh*ns*na
integer :: zmapt(nit,6), idt

!value function
double precision, allocatable :: V(:,:,:,:,:,:)

!temporary value
double precision :: vtemp
double precision :: a1, a2

!variables to simulate distribution
integer :: hcc, scc, acc
double precision :: help(ne,nh,J), dummy, dummyg
double precision, allocatable :: PhiD(:,:,:,:,:,:), PhiD0(:,:,:,:,:,:)
double precision, allocatable :: cfun(:,:,:,:,:,:), lfun(:,:,:,:,:,:), medicaidexpfun(:,:,:,:,:,:), netfun(:,:,:,:,:,:)
double precision, allocatable :: PhiDadjusted(:,:,:,:,:,:,:), afun(:,:,:,:,:,:,:)
integer(kind=1), allocatable :: g_dg(:,:,:,:,:,:,:), g_df(:,:,:,:,:,:,:), g_dm(:,:,:,:,:,:,:)

!variables to compute transfers and employer insurance premiums
double precision, dimension(ne,nh,J) :: Trn
double precision :: transfers, tempy

!variables to compute insurance expense and insurance head count distribution during transition
double precision :: expense(6), store(6)
integer :: tempint
integer :: ja

double precision :: dummyins(ne,J)

npt=(nit-1)/nproc+1
itopt=rank*npt+1
iendt=MIN((rank+1)*npt,nit)

allocate (g_df(ng,ne,nh,ns,na,J,time-1),g_dg(ng,ne,nh,ns,na,J,time-1),g_dm(ng,ne,nh,ns,na,J,time-1))
!initialize for first period
g_df(1:ng,1:ne,1:nh,1:ns,1:na,1:J,1)=g_dfss1
g_dg(1:ng,1:ne,1:nh,1:ns,1:na,1:J,1)=g_dgss1
g_dm(1:ng,1:ne,1:nh,1:ns,1:na,1:J,1)=g_dmss1

allocate (V(ng,ne,nh,ns,na,J+1))
V=0

!parallelizing to backout value function
idt=0
    do tc=1,J
    do gc=1,ng
    do ec=1,ne
    do hc=1,nh
    do sc=1,ns
    do ac=1,na
    idt=idt+1
    zmapt(idt,1)=tc
    zmapt(idt,2)=gc
    zmapt(idt,3)=ec
    zmapt(idt,4)=hc
    zmapt(idt,5)=sc
    zmapt(idt,6)=ac
    end do
    end do
    end do
    end do
    end do
    end do
	
    if ( rank .eq. 0 ) then
!       open(unit=1,file='/panfs/roc/scratch/rave0049/1w/storefun.txt')
!       open(unit=2,file='/panfs/roc/scratch/rave0049/1w/storeafun.txt')
        open(unit=1,file='storefun.txt')
        open(unit=2,file='storeafun.txt')
    end if

allocate (cmpi(Npt),lmpi(Npt),ampi(Npt),vmpi(Npt),medicaidmpi(Npt),netmpi(Npt))
allocate (crecmpi(Npt*nproc),lrecmpi(Npt*nproc),arecmpi(Npt*nproc),vrecmpi(Npt*nproc),medicaidrecmpi(Npt*nproc),netrecMPI(Npt*nproc))
allocate (dfmpi(npt),dgmpi(npt),dmmpi(npt))
allocate (dfrecmpi(npt*nproc),dgrecmpi(npt*nproc),dmrecmpi(npt*nproc))

    !BEGIN BACKWARD INDUCTION

    do date=time-1,2,-1

        if ( date .eq. time-1 ) then
        Vp=Vtime
        else
        Vp=V
        end if
	
	!price of private insurance if allowed to condition on health
        if ( condition(date) .eq. 1 ) then
        pr_g(:,:,:,date)=0

            do ec=1,ne
            do hc=1,nh
            do tc=1,J-1
                if ( tc .lt. jr-1 ) then
                pr_g(ec,hc,tc,date)=cov_p(hc)*psi_t(ec,hc,tc,date)*sum(pi_h(ec,tc,1:nh,hc)*e_h(1:nh,tc+1))/(1+r(date+1))
                else if ( tc .ge. jr-1 ) then
                pr_g(ec,hc,tc,date)=cov_p(hc)*psi_t(ec,hc,tc,date)*(1-cov_m(date+1))*sum(pi_h(ec,tc,1:nh,hc)*e_h(1:nh,tc+1))/(1+r(date+1))
                end if
            end do
            end do
            end do
		
        end if
            
    !restart value function and policy functions
    V=0
    !assume all agents do not choose foodstamp
    G_df(1:ng,1:ne,1:nh,1:ns,1:na,1:J,date)=1
    !assume all agents want to buy
    G_dg(1:ng,1:ne,1:nh,1:ns,1:na,1:J,date)=1
    !assume no Medicaid
    G_dm(1:ng,1:ne,1:nh,1:ns,1:na,1:J,date)=0

    ! Compute optimal policies for Households give date

        do idt=itopt,iendt
        tc=zmapt(idt,1)
        gc=zmapt(idt,2)
        ec=zmapt(idt,3)
        hc=zmapt(idt,4)
        sc=zmapt(idt,5)
        ac=zmapt(idt,6)
	
	!transfers from medicaid and ACA
        medicaidmpi(idt-itopt+1)=zero
        netmpi(idt-itopt+1)=0
		
            if ( hc .gt. sicklevel ) then
            sick=1
            else
            sick=0
            end if

        economy=economy_t(tc,date)

            if ( economy .eq. 'u' ) then
            !do not buy private insurance
            g_dg(gc,ec,hc,sc,ac,tc,date)=0
            !no foodstamp
            g_df(gc,ec,hc,sc,ac,tc,date)=1
            !not eligible for medicaid
            g_dm(gc,ec,hc,sc,ac,tc,date)=0
		
            !if agent works
            work=1
            il=1
            amax=grida(1)
            V(gc,ec,hc,sc,ac,tc)=-FCNtransition(amax)
            ampi(idt-itopt+1)=amax
            cmpi(idt-itopt+1)=cmax
            lmpi(idt-itopt+1)=lmax
                                    
                do ja=2,na
                amax=grida(ja)
                vtemp=-FCNtransition(amax)
                                        
                    if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                    V(gc,ec,hc,sc,ac,tc)=vtemp
                    ampi(idt-itopt+1)=amax
                    cmpi(idt-itopt+1)=cmax
                    lmpi(idt-itopt+1)=lmax
                    end if	

                end do
                                    

                do il=2,nl
                do ja=1,na
                amax=grida(ja)
                vtemp=-FCNtransition(amax)
                    
                    if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                    V(gc,ec,hc,sc,ac,tc)=vtemp
                    ampi(idt-itopt+1)=amax
                    cmpi(idt-itopt+1)=cmax
                    lmpi(idt-itopt+1)=lmax
                    end if	
							
                end do
                end do

            !if agent does not work
            work=0
					
                do ja=1,na
                amax=grida(ja)
                vtemp=-FCNtransition(amax)
                                        
                    if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                    V(gc,ec,hc,sc,ac,tc)=vtemp
                    ampi(idt-itopt+1)=amax
                    cmpi(idt-itopt+1)=cmax
                    lmpi(idt-itopt+1)=lmax
                    end if
						
                end do
                                            
            end if

            if ( economy .eq. 'm' ) then
	
                if ( gc .eq. 1 .or. gc .eq. 2 ) then
					
                work=1
                il=1
                buypvt=1
				
                amax=grida(1)
                V(gc,ec,hc,sc,ac,tc)=-FCNtransition(amax)
                ampi(idt-itopt+1)=amax
                cmpi(idt-itopt+1)=cmax
                lmpi(idt-itopt+1)=lmax
                g_dg(gc,ec,hc,sc,ac,tc,date)=1
                g_dm(gc,ec,hc,sc,ac,tc,date)=elig
                medicaidmpi(idt-itopt+1)=elig*xmed*cov_md(date,tc)
                netmpi(idt-itopt+1)=net
				
                    do ja=2,na
                    amax=grida(ja)
                    vtemp=-FCNtransition(amax)
                                    
                        if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                        V(gc,ec,hc,sc,ac,tc)=vtemp
                        ampi(idt-itopt+1)=amax
                        cmpi(idt-itopt+1)=cmax
                        lmpi(idt-itopt+1)=lmax
                        g_dg(gc,ec,hc,sc,ac,tc,date)=1
                        g_dm(gc,ec,hc,sc,ac,tc,date)=elig
                        medicaidmpi(idt-itopt+1)=elig*xmed*cov_md(date,tc)
                        netmpi(idt-itopt+1)=net					
                        end if
					
                    end do
					
                    do il=2,nl
                    do ja=1,na
                    amax=grida(ja)
                    vtemp=-FCNtransition(amax)
                                                    
                        if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                        V(gc,ec,hc,sc,ac,tc)=vtemp
                        ampi(idt-itopt+1)=amax
                        cmpi(idt-itopt+1)=cmax
                        lmpi(idt-itopt+1)=lmax
                        g_dg(gc,ec,hc,sc,ac,tc,date)=1
                        g_dm(gc,ec,hc,sc,ac,tc,date)=elig
                        medicaidmpi(idt-itopt+1)=elig*xmed*cov_md(date,tc)
                        netmpi(idt-itopt+1)=net		
                        end if
                    end do
                    end do

                !if agent does not work
                buypvt=1
                work=0
					
                    do ja=1,na
                    amax=grida(ja)
                    vtemp=-FCNtransition(amax)

                        if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                        V(gc,ec,hc,sc,ac,tc)=vtemp
                        ampi(idt-itopt+1)=amax
                        cmpi(idt-itopt+1)=cmax
                        lmpi(idt-itopt+1)=lmax
                        g_dg(gc,ec,hc,sc,ac,tc,date)=1
                        g_dm(gc,ec,hc,sc,ac,tc,date)=elig
                        medicaidmpi(idt-itopt+1)=elig*xmed*cov_md(date,tc)
                        netmpi(idt-itopt+1)=net						
                        end if							
                    end do
	
	
                    if ( allbuy(date) .eq. 0 .or. tc .ge. jr ) then
                    
                    buypvt=0
                    work=0
				
                        do ja=1,na
                                    
                        amax=grida(ja)
                        vtemp=-FCNtransition(amax)	
				
                            if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                            V(gc,ec,hc,sc,ac,tc)=vtemp
                            ampi(idt-itopt+1)=amax
                            cmpi(idt-itopt+1)=cmax
                            lmpi(idt-itopt+1)=lmax
                            g_dg(gc,ec,hc,sc,ac,tc,date)=0
                            g_dm(gc,ec,hc,sc,ac,tc,date)=elig
                            medicaidmpi(idt-itopt+1)=elig*xmed*cov_md(date,tc)
                            netmpi(idt-itopt+1)=net					
                            end if
					
                        end do                                    
					
                        
                    buypvt=0
                    work=1

                        do il=1,nl
                        do ja=1,na
                            
                        amax=grida(ja)
                        vtemp=-FCNtransition(amax)
						
                            if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                            V(gc,ec,hc,sc,ac,tc)=vtemp
                            ampi(idt-itopt+1)=amax
                            cmpi(idt-itopt+1)=cmax
                            lmpi(idt-itopt+1)=lmax
                            g_dg(gc,ec,hc,sc,ac,tc,date)=0
                            g_dm(gc,ec,hc,sc,ac,tc,date)=elig
                            medicaidmpi(idt-itopt+1)=elig*xmed*cov_md(date,tc)
                            netmpi(idt-itopt+1)=net							
                            end if
                                                    
                        end do
                        end do
							
                    end if
                    
                else if ( gc .eq. 3 ) then
	
                buypvt=0
                work=1
                il=1
					
                amax=grida(1)
                V(gc,ec,hc,sc,ac,tc)=-FCNtransition(amax)	
                ampi(idt-itopt+1)=amax
                cmpi(idt-itopt+1)=cmax
                lmpi(idt-itopt+1)=lmax
                g_dg(gc,ec,hc,sc,ac,tc,date)=0
                g_dm(gc,ec,hc,sc,ac,tc,date)=elig
                medicaidmpi(idt-itopt+1)=elig*xmed*cov_md(date,tc)
                netmpi(idt-itopt+1)=net

                    do ja=2,na
                    amax=grida(ja)
                    vtemp=-FCNtransition(amax)

                        if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                        V(gc,ec,hc,sc,ac,tc)=vtemp
                        ampi(idt-itopt+1)=amax
                        cmpi(idt-itopt+1)=cmax
                        lmpi(idt-itopt+1)=lmax
                        g_dg(gc,ec,hc,sc,ac,tc,date)=0
                        g_dm(gc,ec,hc,sc,ac,tc,date)=elig
                        medicaidmpi(idt-itopt+1)=elig*xmed*cov_md(date,tc)
                        netmpi(idt-itopt+1)=net
                        end if
						
                    end do
					
                    do il=2,nl
                    do ja=1,na
                    amax=grida(ja)
                    vtemp=-FCNtransition(amax)
                                            
                        if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                        V(gc,ec,hc,sc,ac,tc)=vtemp
                        ampi(idt-itopt+1)=amax
                        cmpi(idt-itopt+1)=cmax
                        lmpi(idt-itopt+1)=lmax
                        g_dg(gc,ec,hc,sc,ac,tc,date)=0
                        g_dm(gc,ec,hc,sc,ac,tc,date)=elig
                        medicaidmpi(idt-itopt+1)=elig*xmed*cov_md(date,tc)
                        netmpi(idt-itopt+1)=net
                        end if
							
                    end do
                    end do

                buypvt=0
                work=0
					
                    do ja=1,na
                    amax=grida(ja)
                    vtemp=-FCNtransition(amax)
                    
                        if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                        V(gc,ec,hc,sc,ac,tc)=vtemp
                        ampi(idt-itopt+1)=amax
                        cmpi(idt-itopt+1)=cmax
                        lmpi(idt-itopt+1)=lmax
                        g_dg(gc,ec,hc,sc,ac,tc,date)=0
                        g_dm(gc,ec,hc,sc,ac,tc,date)=elig
                        medicaidmpi(idt-itopt+1)=elig*xmed*cov_md(date,tc)
                        netmpi(idt-itopt+1)=net
                        end if
						
                    end do
                                                                                        
                    if ( tc .ge. jr-1 ) then

                    buypvt=1
                    work=1
                        
                        do il=1,nl
                        do ja=1,na
                        amax=grida(ja)
                        vtemp=-FCNtransition(amax)
                                                        
                            if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                            V(gc,ec,hc,sc,ac,tc)=vtemp
                            ampi(idt-itopt+1)=amax
                            cmpi(idt-itopt+1)=cmax
                            lmpi(idt-itopt+1)=lmax
                            g_dg(gc,ec,hc,sc,ac,tc,date)=1
                            g_dm(gc,ec,hc,sc,ac,tc,date)=elig
                            medicaidmpi(idt-itopt+1)=elig*xmed*cov_md(date,tc)
                            netmpi(idt-itopt+1)=net
                            end if
								
                        end do
                        end do

                    buypvt=1
                    work=0
						
                        do ja=1,na
                        amax=grida(ja)
                        vtemp=-FCNtransition(amax)
                        
                            if ( vtemp .gt. V(gc,ec,hc,sc,ac,tc) ) then
                            V(gc,ec,hc,sc,ac,tc)=vtemp
                            ampi(idt-itopt+1)=amax
                            cmpi(idt-itopt+1)=cmax
                            lmpi(idt-itopt+1)=lmax
                            g_dg(gc,ec,hc,sc,ac,tc,date)=1
                            g_dm(gc,ec,hc,sc,ac,tc,date)=elig
                            medicaidmpi(idt-itopt+1)=elig*xmed*cov_md(date,tc)
                            netmpi(idt-itopt+1)=net
                            end if
							
                        end do
                                                
                    end if
					
                end if

            !foodstamps
                if ( gc .eq. 1 .or. gc .eq. 2 ) then
                gcp=2
                else if ( gc .eq. 3 ) then
                gcp=3
                end if

                call basefun(grida,na,grida(1),vals,inds)

                do ih=1,nh
                    do is=1,ns
                    VM(ih,is)=vals(1)*Vp(gcp,ec,ih,is,inds(1),tc+1)+vals(2)*Vp(gcp,ec,ih,is,inds(2),tc+1)
                    enddo
                temp(ih)=sum(VM(ih,1:ns)*pi(ec,1:ns,sc))
                end do

            vnotpay=U(cmin,zero,phi_h(hc))+bbeta*psi_t(ec,hc,tc,date)*sum(temp(1:nh)*pi_h(ec,tc,1:nh,hc))

                if ( vnotpay .gt. V(gc,ec,hc,sc,ac,tc) ) then
                G_df(gc,ec,hc,sc,ac,tc,date)=0
                g_dg(gc,ec,hc,sc,ac,tc,date)=0
                ampi(idt-itopt+1)=grida(1)
                cmpi(idt-itopt+1)=cmin
                lmpi(idt-itopt+1)=zero
                V(gc,ec,hc,sc,ac,tc)=vnotpay
                g_dm(gc,ec,hc,sc,ac,tc,date)=0
                medicaidmpi(idt-itopt+1)=0
                netmpi(idt-itopt+1)=0
                end if
				
            end if

            if ( cmpi(idt-itopt+1) .le. zero ) then
            print*,'Consumption negative house.f90, c, gc, ec, hc, sc, ac, tc, date '
            print*,cmpi(idt-itopt+1),gc, ec,hc,sc,ac,tc,date
                call exit
            elseif ( ( lmpi(idt-itopt+1) .lt. zero ) .or. ( lmpi(idt-itopt+1) .ge. (lendowment) ) ) then
            print*, 'Weird labor house.f90', lmpi(idt-itopt+1),gc,ec,hc,sc,ac,tc
                    call exit
            endif

        !for gathering
        vmpi(idt-itopt+1)=V(gc,ec,hc,sc,ac,tc)
        dfmpi(idt-itopt+1)=G_df(gc,ec,hc,sc,ac,tc,date)
        dgmpi(idt-itopt+1)=G_dg(gc,ec,hc,sc,ac,tc,date)
        dmmpi(idt-itopt+1)=G_dm(gc,ec,hc,sc,ac,tc,date)
        end do

        call MPI_ALLGATHER (vmpi,npt,mpi_double_precision,vrecmpi,npt,mpi_double_precision,mpi_comm_world,ierr)
        call MPI_ALLGATHER (cmpi,npt,mpi_double_precision,crecmpi,npt,mpi_double_precision,mpi_comm_world,ierr)
        call MPI_ALLGATHER (lmpi,npt,mpi_double_precision,lrecmpi,npt,mpi_double_precision,mpi_comm_world,ierr)
        call MPI_ALLGATHER (ampi,npt,mpi_double_precision,arecmpi,npt,mpi_double_precision,mpi_comm_world,ierr)
        call MPI_ALLGATHER (dfmpi,npt,mpi_integer1,dfrecmpi,npt,mpi_integer1,mpi_comm_world,ierr)
        call MPI_ALLGATHER (dgmpi,npt,mpi_integer1,dgrecmpi,npt,mpi_integer1,mpi_comm_world,ierr)
        call MPI_ALLGATHER (dmmpi,npt,mpi_integer1,dmrecmpi,npt,mpi_integer1,mpi_comm_world,ierr)
        call MPI_ALLGATHER (medicaidmpi,npt,mpi_double_precision,medicaidrecmpi,npt,mpi_double_precision,mpi_comm_world,ierr)
        call MPI_ALLGATHER (netmpi,npt,mpi_double_precision,netrecmpi,npt,mpi_double_precision,mpi_comm_world,ierr)

    idt=0
		
        do tc=1,J
        do gc=1,ng
        do ec=1,ne
        do hc=1,nh
        do sc=1,ns
        do ac=1,na
        idt=idt+1
        V(gc,ec,hc,sc,ac,tc)=vrecmpi(idt)
        g_df(gc,ec,hc,sc,ac,tc,date)=dfrecmpi(idt)
        g_dg(gc,ec,hc,sc,ac,tc,date)=dgrecmpi(idt)
        g_dm(gc,ec,hc,sc,ac,tc,date)=dmrecmpi(idt)
								
            if ( rank .eq. 0 ) then
            write(1,*)crecmpi(idt),lrecmpi(idt),medicaidrecmpi(idt),netrecmpi(idt)
            write(2,*)arecmpi(idt)
            end if
									
        end do
        end do
        end do
        end do
        end do
        end do

        if ( date .eq. 3 ) then
        g_dmss3=g_dm(:,:,:,:,:,:,3)
        g_dfss3=g_df(:,:,:,:,:,:,3)
        end if
        
        if ( date .eq. 2 ) then
        g_dfss2=g_df(:,:,:,:,:,:,2)
        end if

    Vborn(:,:,:,:,date)=V(:,:,:,:,1,1)
        
    end do

    !DONE WITH BACKWARD INDUCTION

    if ( rank .eq. 0 ) then
        close(1)
        close(2)
    print*,'Done with backward induction ',date
    end if
	
    call MPI_BARRIER(mpi_comm_world,ierr)
	
 deallocate (cmpi,lmpi,ampi,vmpi,medicaidmpi,netMPI)
 deallocate (crecmpi,lrecmpi,arecmpi,vrecmpi,medicaidrecmpi,netrecMPI)
 deallocate (dfmpi,dgmpi,dmmpi)
 deallocate (dfrecmpi,dgrecmpi,dmrecmpi) 	 

 !saving these variables to use in voting.f90 and welfare.f90
 V2=V
 deallocate(V)
 
    !BEGIN SIMULATION
	
    allocate (PhiD0(ng,ne,nh,ns,na,J),Phid(ng,ne,nh,ns,na,J),Phidadjusted(ng,ne,nh,ns,na,J,time-1))
		
    PhiD0(1:ng,1:ne,1:nh,1:ns,1:na,1:J)=Phiss1
    PhiDadjusted(1:ng,1:ne,1:nh,1:ns,1:na,1:J,1)=Phissadjusted1

    allocate (afun(ng,ne,nh,ns,na,J,time-1))
    afun(1:ng,1:ne,1:nh,1:ns,1:na,1:J,1)=afunss1

	
!   open(unit=2,file='/panfs/roc/scratch/rave0049/1w/storeafun.txt')
    open(unit=2,file='storeafun.txt')
	
        do date=time-1,2,-1
        do tc=1,J
        do gc=1,ng
        do ec=1,ne
        do hc=1,nh
        do sc=1,ns
        do ac=1,na
        read(2,*)afun(gc,ec,hc,sc,ac,tc,date)
        end do
        end do
        end do
        end do
        end do
        end do
        end do
        
    close(2)

	
    do date=2,time-1

    ! Initialize Distribution by Computing Distribution for first Generation
    PhiD=0

        do gc=1,ng
        do ec=1,ne
        do hc=1,nh	
        do sc=1,ns
        PhiD(gc,ec,hc,sc,1,1)=put_h(ec,1,hc)*put_e_t(ec,date)*put(ec,sc)*put_g(gc)
        end do
        end do
        end do	
        end do

    ! Loop to find distributions for generations 2 to J

        do tc=2,J
        do gc=1,ng
        do ec=1,ne
        do hc=1,nh
        do sc=1,ns				! Loop over States of productivity
        do ac=1,na				! Loop over old assets
							
            call basefun(grida,na,afun(gc,ec,hc,sc,ac,tc-1,date-1),vals,inds)
							
            do hcc=1,nh
            do scc=1,ns

                if ( g_dg(gc,ec,hc,sc,ac,tc-1,date-1) .eq. 1 .and. ( gc .eq. 1 .or. gc .eq. 2 ) ) then 
						
                PhiD(1,ec,hcc,scc,inds(1),tc)=PhiD(1,ec,hcc,scc,inds(1),tc)+&
                PhiD0(gc,ec,hc,sc,ac,tc-1)*vals(1)*pi(ec,scc,sc)*pi_h(ec,tc-1,hcc,hc)
                PhiD(1,ec,hcc,scc,inds(2),tc)=PhiD(1,ec,hcc,scc,inds(2),tc)+&
                PhiD0(gc,ec,hc,sc,ac,tc-1)*vals(2)*pi(ec,scc,sc)*pi_h(ec,tc-1,hcc,hc)
						
                else if ( g_dg(gc,ec,hc,sc,ac,tc-1,date-1) .eq. 0 .and. ( gc .eq. 1 .or. gc .eq. 2 ) ) then 
						
                PhiD(2,ec,hcc,scc,inds(1),tc)=PhiD(2,ec,hcc,scc,inds(1),tc)+&
                PhiD0(gc,ec,hc,sc,ac,tc-1)*vals(1)*pi(ec,scc,sc)*pi_h(ec,tc-1,hcc,hc)
                PhiD(2,ec,hcc,scc,inds(2),tc)=PhiD(2,ec,hcc,scc,inds(2),tc)+&
                PhiD0(gc,ec,hc,sc,ac,tc-1)*vals(2)*pi(ec,scc,sc)*pi_h(ec,tc-1,hcc,hc)
						
                else if ( gc .eq. 3 .and. g_dg(gc,ec,hc,sc,ac,tc-1,date-1) .eq. 1 ) then 
                                
                PhiD(1,ec,hcc,scc,inds(1),tc)=PhiD(1,ec,hcc,scc,inds(1),tc)+&
                PhiD0(gc,ec,hc,sc,ac,tc-1)*vals(1)*pi(ec,scc,sc)*pi_h(ec,tc-1,hcc,hc)
                PhiD(1,ec,hcc,scc,inds(2),tc)=PhiD(1,ec,hcc,scc,inds(2),tc)+&
                PhiD0(gc,ec,hc,sc,ac,tc-1)*vals(2)*pi(ec,scc,sc)*pi_h(ec,tc-1,hcc,hc)

                else if ( gc .eq. 3 .and. g_dg(gc,ec,hc,sc,ac,tc-1,date-1) .eq. 0 ) then 
						
                PhiD(3,ec,hcc,scc,inds(1),tc)=PhiD(3,ec,hcc,scc,inds(1),tc)+&
                PhiD0(gc,ec,hc,sc,ac,tc-1)*vals(1)*pi(ec,scc,sc)*pi_h(ec,tc-1,hcc,hc)
                PhiD(3,ec,hcc,scc,inds(2),tc)=PhiD(3,ec,hcc,scc,inds(2),tc)+&
                PhiD0(gc,ec,hc,sc,ac,tc-1)*vals(2)*pi(ec,scc,sc)*pi_h(ec,tc-1,hcc,hc)
						
                end if

            end do
            end do
            
        end do
        end do
        end do
        end do
        end do
        end do

    !END SIMULATION
    
        do tc=1,J
        do ec=1,ne
        do hc=1,nh
        dummy=sum(PhiD(:,ec,hc,:,:,tc))
            do gc=1,ng
            do sc=1,ns
            do ac=1,na
                if ( dummy .gt. zero ) then
                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)=PhiD(gc,ec,hc,sc,ac,tc)/dummy
                else
                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)=0
                end if
            end do		
            end do
            end do
        end do
        end do	
        end do

    PhiD0=PhiD
    Adis(:,date)=0

        do ac=1,na
        do tc=1,J
        do ec=1,ne
        do hc=1,nh
        help(ec,hc,tc)=sum(PhiDadjusted(:,ec,hc,:,ac,tc,date))
        ADis(ac,date)=ADis(ac,date)+help(ec,hc,tc)*Nu(ec,hc,tc,date)/sum(Nu(:,:,:,date))
        end do
        end do
        end do
        end do

        if ( ADis(na,date) .gt. zero .and. rank .eq. 0 ) then
        print*,'Enlarge Grid (printed from house.f90)', ADis(na,date)
!	call exit
        end if

        if ( sum(ADis(1:na,date)) > 1.01d0 .and. rank .eq. 0 ) then
        print*,'Should equal 1', sum(ADis(1:na,date))
            call exit
        end if
		

    As(date)=0	
    Trn=0
    transfers=0
	
        do tc=1,J
        do ec=1,ne
        do hc=1,nh	
        
            do gc=1,ng
            do sc=1,ns
            As(date)=As(date)+Nu(ec,hc,tc,date)*(sum(PhiDadjusted(gc,ec,hc,sc,1:na,tc,date)*afun(gc,ec,hc,sc,1:na,tc,date))+& 
            pr_g(ec,hc,tc,date)*sum(g_dg(gc,ec,hc,sc,1:na,tc,date)*PhiDadjusted(gc,ec,hc,sc,1:na,tc,date)))						
            
            Trn(ec,hc,tc)=Trn(ec,hc,tc)+sum(afun(gc,ec,hc,sc,1:na,tc,date-1)*PhiDadjusted(gc,ec,hc,sc,1:na,tc,date-1))
            end do
            end do
	
        transfers=transfers+(1-psi_t(ec,hc,tc,date-1))*Nu(ec,hc,tc,date-1)*Trn(ec,hc,tc)
				
        end do
        end do
        end do

    !transfers of assets of dead,
    Tr1(date)=transfers/(sum(Nu(1:ne,1:nh,1:J,date))*(1+nn))	
	
        if ( date .eq. 3 ) then

        PhiDadjusted3=phiDadjusted(:,:,:,:,:,:,date)        
        Phissage3=0

            do tc=1,J
 !           dummy=sum(phiD(:,:,:,:,:,tc))
                do gc=1,ng
                do ec=1,ne
                do hc=1,nh
                do sc=1,ns
                do ac=1,na
!                    if ( dummy .gt. zero ) then
                phissage3(gc,ec,hc,sc,ac,tc)=phiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/sum(Nu(:,:,tc,date))
                    !/dummy
!                    else
!                    phissage3(gc,ec,hc,sc,ac,tc)=0
!                    end if
                end do		
                end do
                end do
                end do	
                end do
		end do

        end if

    end do
	
    deallocate(Phid,PhiD0)
    deallocate(afun)

insexptran(1,:)=insexp(1:6)
insexptran(time,:)=insexp(7:12)
			
    allocate (cfun(ng,ne,nh,ns,na,J),lfun(ng,ne,nh,ns,na,J),medicaidexpfun(ng,ne,nh,ns,na,J),netfun(ng,ne,nh,ns,na,J))
	
!open (unit=1,file='/panfs/roc/scratch/rave0049/1w/storefun.txt')
    open (unit=1,file='storefun.txt')
        
        do date=time-1,2,-1


        C(date)=0		
        H(date)=0
        hour(date)=0
        N1(date)=0
		
        safety(date)=0
        safetycnt(date)=0

        !insurance expense and head count distribution during transition
        insexptran(date,:)=0

        !private, self, employer, Medicare, Welfare, Medicaid
        pvtinsured(date)=0
        selfinsured(date)=0
        employer(date)=0
        medicare(date)=0
        foodstamp(date)=0
        medicaid(date)=0

        dummy=0     !employment insurance health expense
        tempy=0
        dummyg=0    !government expenditure
        dummyins=0  !mass of agents buying private insurance given ec and tc
		
            if ( condition(date) .eq. 0 ) then
            pr_g1(:,:,:,date)=zero
            end if
		
            do tc=1,J
            economy=economy_t(tc,date)

                do gc=1,ng
                do ec=1,ne
                do hc=1,nh
						
                    if ( hc .gt. sicklevel ) then
                    sick=1
                    else	
                    sick=0
                    end if
						
                    do sc=1,ns
                    do ac=1,na
								
                    read(1,*)cfun(gc,ec,hc,sc,ac,tc),lfun(gc,ec,hc,sc,ac,tc),medicaidexpfun(gc,ec,hc,sc,ac,tc),netfun(gc,ec,hc,sc,ac,tc)
								
                        if ( condition(date) .eq. 0 ) then
                    
                            if ( tc .lt. jr-1 ) then

                            pr_g1(ec,1,tc,date)=pr_g1(ec,1,tc,date)+G_dg(gc,ec,hc,sc,ac,tc,date)*&
                            psi_t(ec,hc,tc,date)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)*&
                            cov_p(hc)*sum(pi_h(ec,tc,1:nh,hc)*e_h(1:nh,tc+1))   

                            dummyins(ec,tc)=dummyins(ec,tc)+G_dg(gc,ec,hc,sc,ac,tc,date)*&
                            PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)	

                            else if ( tc .ge. jr-1 .and. tc .lt. J ) then
                            
                            pr_g1(ec,1,tc,date)=pr_g1(ec,1,tc,date)+G_dg(gc,ec,hc,sc,ac,tc,date)*&
                            psi_t(ec,hc,tc,date)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)*&
                            (1-cov_m(date+1))*cov_p(hc)*sum(pi_h(ec,tc,1:nh,hc)*e_h(1:nh,tc+1))   
                            dummyins(ec,tc)=dummyins(ec,tc)+G_dg(gc,ec,hc,sc,ac,tc,date)*&
                            PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)	                                                                                
                            end if                    

                        else if ( condition(date) .eq. 1 ) then                                                                        
                                                                    
                        pr_g1(ec,hc,tc,date)=pr_g(ec,hc,tc,date)
                                                                        
                        end if
                               
                        !calculate employment insurance health expense
                        if ( tc .lt. jr .and. gc .eq. 3 .and. economy .eq. 'm' ) then									
									
                            if ( G_df(3,ec,hc,sc,ac,tc,date) .eq. 1 ) then
                            dummy=dummy+cov_e(hc)*e_h(hc,tc)*PhiDadjusted(3,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)	
													
                                if ( lfun(3,ec,hc,sc,ac,tc) .gt. zero ) then 
                                tempy=tempy+PhiDadjusted(3,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date) 
                                end if							
                                
                            end if
											
                        end if							

                        if ( economy .eq. 'u' ) then
										
                        dummyg=dummyg+e_h(hc,tc)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date) 
									
                        else if ( economy .eq. 'm' ) then

                        !Medicare
                            if ( tc .ge. jr ) then
                            dummyg=dummyg+cov_m(date)*e_h(hc,tc)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)  
                            end if

                        !Medicaid
                        dummyg=dummyg+medicaidexpfun(gc,ec,hc,sc,ac,tc)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date) 
                        safetycnt(date)=safetycnt(date)+(1-G_df(gc,ec,hc,sc,ac,tc,date))*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*nu(ec,hc,tc,date)/sum(nu(:,:,:,date))

                        dummyg=dummyg+netfun(gc,ec,hc,sc,ac,tc)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
									
                            !foodstamp
                            if ( tc .lt. jr ) then

                                if ( gc .eq. 2 .or. gc .eq. 3 ) then
                                dummyg=dummyg+(1-G_df(gc,ec,hc,sc,ac,tc,date))*&
                                (e_h(hc,tc)+cmin-(1+r(date)*(1-tauk(date)))&
                                *(grida(ac)+Tr(date)))*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date) 

                                safety(date)=safety(date)+(1-G_df(gc,ec,hc,sc,ac,tc,date))*&
                                (e_h(hc,tc)+cmin-(1+r(date)*(1-tauk(date)))&
                                *(grida(ac)+Tr(date)))*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*&
                                nu(ec,hc,tc,date)

                                else if ( gc .eq. 1 ) then
                                dummyg=dummyg+(1-G_df(gc,ec,hc,sc,ac,tc,date))*&
                                ((1-cov_p(hc))*e_h(hc,tc)+cmin-(1+r(date)*(1-tauk(date)))&
                                *(grida(ac)+Tr(date)))*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date) 

                                safety(date)=safety(date)+(1-G_df(gc,ec,hc,sc,ac,tc,date))*&
                                ((1-cov_p(hc))*e_h(hc,tc)+cmin-(1+r(date)*(1-tauk(date)))&
                                *(grida(ac)+Tr(date)))*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*&
                                nu(ec,hc,tc,date)

                                end if 
				
                            else if ( tc .ge. jr ) then

                                if ( gc .eq. 2 .or. gc .eq. 3 ) then
                                dummyg=dummyg+(1-G_df(gc,ec,hc,sc,ac,tc,date))*&
                                ((1-cov_m(date))*e_h(hc,tc)+cmin-(1+r(date)*(1-tauk(date)))&
                                *(grida(ac)+Tr(date))-SS(ec,date))*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date) 

                                safety(date)=safety(date)+(1-G_df(gc,ec,hc,sc,ac,tc,date))*&
                                ((1-cov_m(date))*e_h(hc,tc)+cmin-(1+r(date)*(1-tauk(date)))&
                                *(grida(ac)+Tr(date))-SS(ec,date))*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*nu(ec,hc,tc,date)

                                else if ( gc .eq. 1 ) then
                                dummyg=dummyg+(1-G_df(gc,ec,hc,sc,ac,tc,date))*&
                                ((1-cov_p(hc))*(1-cov_m(date))*e_h(hc,tc)+&
                                cmin-(1+r(date)*(1-tauk(date)))*&
                                (grida(ac)+Tr(date))-SS(ec,date))*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date) 

                                safety(date)=safety(date)+(1-G_df(gc,ec,hc,sc,ac,tc,date))*&
                                ((1-cov_p(hc))*(1-cov_m(date))*e_h(hc,tc)+&
                                cmin-(1+r(date)*(1-tauk(date)))*&
                                (grida(ac)+Tr(date))-SS(ec,date))*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*nu(ec,hc,tc,date)

                                end if
                            end if

                        end if

                    store(:)=insexptran(date,:)

                        if ( tc .lt. jr ) then
                        
                            if ( gc .eq. 1 ) then

                            insexptran(date,1)=insexptran(date,1)+e_h(hc,tc)*cov_p(hc)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)

                                if ( g_dm(gc,ec,hc,sc,ac,tc,date) .eq. 1 ) then
                                                                    
                                insexptran(date,6)=insexptran(date,6)+medicaidexpfun(gc,ec,hc,sc,ac,tc)&
                                *PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)

                                insexptran(date,2)=insexptran(date,2)+medicaidexpfun(gc,ec,hc,sc,ac,tc)*(1-cov_md(date,tc))&
                                *PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/cov_md(date,tc)

                                else if ( g_df(gc,ec,hc,sc,ac,tc,date) .eq. 0 ) then
                                insexptran(date,5)=insexptran(date,5)+e_h(hc,tc)*(1-cov_p(hc))*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
                                else
                                insexptran(date,2)=insexptran(date,2)+e_h(hc,tc)*(1-cov_p(hc))*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
                                end if

                            else if ( gc .eq. 2 ) then

                                if ( g_dm(gc,ec,hc,sc,ac,tc,date) .eq. 1 ) then
                                insexptran(date,6)=insexptran(date,6)+&
                                medicaidexpfun(gc,ec,hc,sc,ac,tc)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)

                                insexptran(date,2)=insexptran(date,2)+medicaidexpfun(gc,ec,hc,sc,ac,tc)*(1-cov_md(date,tc))&
                                *PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/cov_md(date,tc)

                                else if ( g_df(gc,ec,hc,sc,ac,tc,date) .eq. 0 ) then
                                insexptran(date,5)=insexptran(date,5)+e_h(hc,tc)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
                                else
                                insexptran(date,2)=insexptran(date,2)+e_h(hc,tc)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
                                end if

                            else if ( gc .eq. 3 ) then

                                if ( g_dm(gc,ec,hc,sc,ac,tc,date) .eq. 1 ) then
                                
                                insexptran(date,3)=insexptran(date,3)+e_h(hc,tc)*cov_e(hc)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)

                                insexptran(date,6)=insexptran(date,6)+medicaidexpfun(gc,ec,hc,sc,ac,tc)&
                                *PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)

                                insexptran(date,2)=insexptran(date,2)+medicaidexpfun(gc,ec,hc,sc,ac,tc)*(1-cov_md(date,tc))&
                                *PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/cov_md(date,tc)

                                else if ( g_df(gc,ec,hc,sc,ac,tc,date) .eq. 0 ) then
                                insexptran(date,5)=insexptran(date,5)+e_h(hc,tc)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
                                else
                                insexptran(date,3)=insexptran(date,3)+e_h(hc,tc)*cov_e(hc)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
                                insexptran(date,2)=insexptran(date,2)+e_h(hc,tc)*(1-cov_e(hc))*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
                                end if

                            end if

                        else if ( tc .ge. jr ) then

                        insexptran(date,4)=insexptran(date,4)+e_h(hc,tc)*cov_m(date)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
	
                            if ( gc .eq. 1 ) then
                            insexptran(date,1)=insexptran(date,1)+(1-cov_m(date))*e_h(hc,tc)*cov_p(hc)*PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)

                                if ( g_dm(gc,ec,hc,sc,ac,tc,date) .eq. 1 ) then
												
                                insexptran(date,6)=insexptran(date,6)+medicaidexpfun(gc,ec,hc,sc,ac,tc)*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)

                                insexptran(date,2)=insexptran(date,2)+medicaidexpfun(gc,ec,hc,sc,ac,tc)*(1-cov_md(date,tc))&
                                *PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/cov_md(date,tc)
				
                                else if ( g_df(gc,ec,hc,sc,ac,tc,date) .eq. 0 ) then

                                insexptran(date,5)=insexptran(date,5)+(1-cov_m(date))*e_h(hc,tc)*(1-cov_p(hc))*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)

                                else

                                insexptran(date,2)=insexptran(date,2)+(1-cov_m(date))*e_h(hc,tc)*(1-cov_p(hc))*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)

                                end if

                            else if ( gc .eq. 2 ) then

                                if ( g_dm(gc,ec,hc,sc,ac,tc,date) .eq. 1 ) then

                                insexptran(date,6)=insexptran(date,6)+medicaidexpfun(gc,ec,hc,sc,ac,tc)*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)

                                insexptran(date,2)=insexptran(date,2)+medicaidexpfun(gc,ec,hc,sc,ac,tc)*(1-cov_md(date,tc))&
                                *PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/cov_md(date,tc)

                                else if ( g_df(gc,ec,hc,sc,ac,tc,date) .eq. 0 ) then
                                insexptran(date,5)=insexptran(date,5)+(1-cov_m(date))*e_h(hc,tc)*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
                                else
                                insexptran(date,2)=insexptran(date,2)+(1-cov_m(date))*e_h(hc,tc)*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
                                end if

                            else if ( gc .eq. 3 ) then

                                if ( g_dm(gc,ec,hc,sc,ac,tc,date) .eq. 1 ) then
                                insexptran(date,6)=insexptran(date,6)+medicaidexpfun(gc,ec,hc,sc,ac,tc)*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)

                                insexptran(date,2)=insexptran(date,2)+medicaidexpfun(gc,ec,hc,sc,ac,tc)*(1-cov_md(date,tc))&
                                *PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/cov_md(date,tc)

                                else if ( g_df(gc,ec,hc,sc,ac,tc,date) .eq. 0 ) then
                                insexptran(date,5)=insexptran(date,5)+(1-cov_m(date))*e_h(hc,tc)*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
                                else
                                insexptran(date,2)=insexptran(date,2)+(1-cov_m(date))*e_h(hc,tc)*&
                                PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)
                                end if
                            end if
                        end if

                    expense(:)=insexptran(date,:)-store(:)
                    tempint=maxloc(expense,1)

                        if ( tempint .eq. 1 ) then
                        pvtinsured(date)=pvtinsured(date)+PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/sum(nu(:,:,:,date))
                        else if ( tempint .eq. 2 ) then
                        selfinsured(date)=selfinsured(date)+PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/sum(nu(:,:,:,date))
                        else if ( tempint .eq. 3 ) then
                        employer(date)=employer(date)+PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/sum(nu(:,:,:,date))
                        else if ( tempint .eq. 4 ) then
                        medicare(date)=medicare(date)+PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/sum(nu(:,:,:,date))
                        else if ( tempint .eq. 5 ) then
                        foodstamp(date)=foodstamp(date)+PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/sum(nu(:,:,:,date))
                        else if ( tempint .eq. 6 ) then
                        medicaid(date)=medicaid(date)+PhiDadjusted(gc,ec,hc,sc,ac,tc,date)*Nu(ec,hc,tc,date)/sum(nu(:,:,:,date))
                        end if
														
                    end do

                    C(date)=C(date)+Nu(ec,hc,tc,date)*sum(PhiDadjusted(gc,ec,hc,sc,1:na,tc,date)*cfun(gc,ec,hc,sc,1:na,tc))
                    H(date)=H(date)+Nu(ec,hc,tc,date)*e_h(hc,tc)*sum(PhiDadjusted(gc,ec,hc,sc,1:na,tc,date))
                    N1(date)=N1(date)+Nu(ec,hc,tc,date)*eta(ec,sc)*ep(ec,tc)*z_h(hc)*sum(PhiDadjusted(gc,ec,hc,sc,1:na,tc,date)*lfun(gc,ec,hc,sc,1:na,tc))
                    hour(date)=hour(date)+Nu(ec,hc,tc,date)*sum(PhiDadjusted(gc,ec,hc,sc,1:na,tc,date)*lfun(gc,ec,hc,sc,1:na,tc))/sum(Nu(:,:,:,date))
								
                    end do

						
                end do
                end do
                end do
                
            end do
			
            if ( condition(date) .eq. 0 ) then
            
                do ec=1,ne
                do tc=1,J-1
                    if ( dummyins(ec,tc) .gt. zero ) then
                    pr_g1(ec,1,tc,date)=pr_g1(ec,1,tc,date)/((1+r(date+1))*dummyins(ec,tc))
                    pr_g1(ec,2:nh,tc,date)=pr_g1(ec,1,tc,date)
                    else
                        if ( rank .eq. 0 ) then
                        print*,'zero purchase of private insurance at house.f90 ',ec,tc
                        end if
                    pr_g1(ec,1,tc,date)=.9*pr_g(ec,1,tc,date)
                    pr_g1(ec,2:nh,tc,date)=pr_g1(ec,1,tc,date)
                    end if
                end do
                end do
                
            end if
                    
!        hour(date)=hour(date)*sum(Nuss)/wapss
        hour(date)=hour(date)*sum(Nu(:,:,:,date))/sum(Nu(:,:,1:jr-1,date))

            if ( dummy .eq. zero ) then
            tauf1(date)=zero
            else
            tauf1(date)=dummy/tempy
            end if
                                
        ghexp1(date)=dummyg
		
        end do
		
    close(1)
	
    deallocate(cfun,lfun,medicaidexpfun,netfun)	
    deallocate(Phidadjusted)
    deallocate(g_df,g_dm,g_dg)
    
    call MPI_BARRIER(mpi_comm_world,ierr)

end subroutine house
