module params

IMPLICIT NONE

double precision, parameter :: y_2010=48374.05646

!storing 0 and 1 for if statements with double precision numbers
double precision, parameter :: zero=0, one=1
! !maximum life span (20 to 86 (die at 87 with probability 1)), retire at 66, periods for transition, number of states for idisyncratic productiviy, number of grid points for assets, number of health states, number of states for education, number of insurance states
! integer,parameter :: J=67, jr=47, maxut=1000, time=100, ns=4, na=101, nh=3, ne=2, ng=3

!maximum life span (20 to 100 (die at 101 with probability 1)), retire at 66, periods for transition, number of states for idisyncratic productiviy, number of grid points for assets, number of health states, number of states for education, number of insurance states
integer,parameter :: J=81, jr=47, maxut=1000, time=100, ns=4, na=251, nh=3, ne=2, ng=3

!number of gridpoints for indivisible labor
integer, parameter :: nl=3
!penalty for negative consumption
double precision,parameter ::umin=-1e35
double precision, parameter :: FPL=0.218528446  !average first person FPL/GDP pc from 2006 to 2010
double precision, parameter :: ycatparam=.9022*FPL   
double precision, parameter :: ymnparam=.419*FPL, amnparam=1949.75/47001.55535

!education: no college and college
!health: good and low, good and high, good and catastrophic, sick and low, sick and high, sick and catastrophic

!economy='m' is one with private insurance, self insurance, employer insurance, Medicaid, and foodstamps+EMTALA+charity
!economy='u' is universal health care (100 percent coverage)
!economy is chosen in policies.f90
!insurance: status 1 is privately insured, status 2 is self insured, status 3 and less than jr is employer insured, status 3 and above jr is self insured

!capital share of income, depreciation, population growth
double precision,parameter :: alpha=0.36, delta=0.059, nn=.011
!labor endowment, full time, part time, extra time
double precision, parameter :: lendowment=1, ft=.3, pt=.75*ft, et=1.25*ft

!determines threshold beyond which agents are sick
integer, parameter :: sicklevel=2

!following parameters are set in calibrate.f90

!discount rate, preference for consumption, risk aversion, tfp
double precision :: beta, gamma, sigma, theta
!minimal consumption provided by government when on foodstamps
double precision :: cmin
!minimum those on employer insurnace have to work
double precision :: lmin
!health productivity shock  
double precision, dimension(nh) :: z_h
!health expenditure shock
double precision, dimension(nh,J) :: e_h
!coverage for private insurance
double precision, dimension(nh) :: cov_p
!coverage for employer insurance
double precision, dimension(nh) :: cov_e
!coverage for Medicare
double precision :: cov_m(time+1), cov_mss
!coverage for Medicaid
double precision :: cov_md(time,J), cov_mdss(J)
!Medicaid categorical variables and medically needy variables
double precision, dimension(time) :: ycat, kcat, ymn, kmn
double precision :: ycatss, kcatss, ymnss, kmnss
!fixed endowment cost of working
double precision :: phi_l
!initial insurance distribution
double precision, dimension(ng) :: put_g
!parameters for earnings process: persistence for normal, persistence at top, probability of going to top
double precision :: rho_c, rho_topc, pi_topc
!productivity at top, dispersion for normal
double precision :: eta_topc, sigma_c
!transition matrix for idiosyncratic productivity
double precision,dimension(ne,ns,ns) :: pi

!grid for idiosyncratic productivity, stationary distribution of idisyncratic productivity (parameterized in markov.f90) 
double precision,dimension(ne,ns) :: eta, put	!put will be calculated given pi

!stationary distribution of health (computed in demographics.f90)
!assume initial distribution of put_h same as put_h at age=21
double precision,dimension(ne,J,nh) :: put_h
!education distribution (computed in demographics.f90)
double precision,dimension(ne) :: put_e
! age efficiency units (parameterized in demographics.f90)
double precision,dimension(ne,J) :: ep

!idiosyncratic productivity count, asset count, age count, health status count, insurance count, time count
integer :: sc, ac, tc, hc, ec, gc, date
!indivisible labor count
integer :: il

!discount factor normalized for population growth, total population in period 1 (parameterized in calibrate.f90)
double precision :: bbeta 

!variables for mpi (parallel processing)
integer :: ierr, rank, nproc

!transition matrix for health (parameterized in demographics.f90)
double precision,dimension(ne,J-1,nh,nh) :: pi_h 

!survival probabilities (parameterized in demographics.f90) 
double precision,dimension(ne,nh,J) :: psi

! Number of Agents in population (parameterized in demographics.f90)
double precision,dimension(ne,nh,J,time) :: Nu

! Fraction of agents in population in period 1 by age (parameterized in demographics.f90)

!initial distribution of agents by type in steady state (parameterized in instst.f90 and fistst.f90)
double precision,dimension(ne,nh,J) :: Nuss

!grid for assets and indivisible labor (grid.f90)
double precision :: grida(na), gridl(nl)

!government consumption, capital tax, labor tax, payroll tax for social security, replacement rate (policies.f90 and calibrate.f90)
double precision, dimension(time) :: gcon, tauk, taul, taup
double precision, dimension(ne,time) :: b

!which economy? 'm' or 'u' (description at top)
character :: economy
!economy determined in policy.f90
character, dimension (J,time) :: economy_t
!economy for steady state (instst.f90 and fistst.f90)
character, dimension(J) :: economyss

!replacement rate in ss, government consumption in ss, capital tax rate in ss, labor tax rate in ss
double precision, dimension(ne) :: bss
double precision :: gconss, taukss, taulss, taupss

!variables specific to steady state (policy functions for steady state: labor, assets for tomorrow, consumption)
double precision,dimension(ng,ne,nh,ns,na,J) :: lfunss, afunss, cfunss
!save initial steadystate variables
double precision,dimension(ng,ne,nh,ns,na,J) :: lfunss1, afunss1, cfunss1
!insurance provider
integer(kind=1),dimension(ng,ne,nh,ns,na,J):: hifunss, hifunss1
!1 is no foodstamp, 0 is foodstamp; for private insurance 1 is buying private insurance, 0 is not buying private insurance
!1 is receiving Medicaid and 0 is not receiving Medicaid
integer(kind=1), dimension(ng,ne,nh,ns,na,J) :: g_dfss, g_dgss, g_dmss
integer(kind=1), dimension(ng,ne,nh,ns,na,J) :: g_dfss1, g_dgss1, g_dmss1

!value function for steady state
double precision,dimension(ng,ne,nh,ns,na,J+1) :: Vss
!distributions for steady state
double precision,dimension(ng,ne,nh,ns,na,J) :: Phiss, Phissadjusted, Phissage, Phisstrue
double precision,dimension(ng,ne,nh,ns,na,J) :: Phiss1

!Aggregate savings for steady state
double precision,dimension(na) :: ADisss
!other steady state variables
double precision :: rss, Nss, N1ss, Kss, wss, Trss, Yss, Asss, Css, hourss, Hss, ghexpss, taufss
double precision, dimension(ne) :: SSss
!price of private insurance
double precision,dimension(ne,nh,J) :: pr_gss

!these variables are to calculate life profile of hours
double precision, dimension(J) :: labarssbyage, savingsbyage, consbyage
double precision, dimension(J) :: cons1byage, savings1byage, labincbyage, savings2byage, nettransfersbyage

!Variables specific of transition
double precision, dimension(ng,ne,nh,ns,na,J+1) :: Vp
double precision, dimension(ne,nh,J,time) :: pr_g, pr_g1

double precision, dimension(na,time) :: ADis
double precision, dimension(time) :: r, r1, N, N1, K, K1, w, Tr, Y, As, C, hour, H, ghexp, tauf
double precision, dimension(time) :: Tr1, ghexp1, tauf1
double precision, dimension(ne,time) :: SS

!Variables for voting.f90 and welfare.f9
double precision, dimension(ng,ne,nh,ns,na,J+1) :: V1, V2, Vtime	
integer, dimension(ng,ne,nh,ns,na,J) :: g_dmss3, g_dmsstime
integer, dimension(ng,ne,nh,ns,na,J) :: g_dfss3, g_dfss2
double precision,dimension(ng,ne,nh,ns,na,J) :: Phissadjusted1, Phissage1, Phissagetime, Phissage3, PhiDadjusted3
integer(kind=1), dimension(ng,ne,nh,ns,na,J) :: quintassetdist, quintlabearningsdist
double precision, dimension(ng,ne,nh,ns,time) :: Vborn



!1 implies work and 0 implies not work
integer :: work
!1 is buy and 0 is not buy
integer :: buypvt
!sick status 
integer :: sick
!eligibility for Medicaid
integer :: elig

!variables used globally for optimization
double precision :: amax, cmax, lmax, xmed, net

!insurance distribution by head count (private, self, employer, Medicare, Welfare, Medicaid)
double precision :: pvtinsuredss, selfinsuredss, employerss, medicaress, foodstampss, medicaidss
double precision, dimension(time) :: pvtinsured, selfinsured, employer, medicare, foodstamp, medicaid

!insurance distribution by headcount (private, self, employer, Medicare, Welfare, Medicaid)
double precision :: ins(12), insage(j,12), insedu(ne,12), insasset(na,12), inshealth(nh,12), insprod(ns,12)
!insurance distribution by expense (private, self, employer, Medicare, Welfare, Medicaid)
double precision :: insexp(12), insexpage(J,12), insexpedu(ne,12), insexpasset(na,12), insexphealth(nh,12), insexpprod(ns,12)
!insurance expense during transition
double precision :: insexptran(time,6)
!spending for foodstamp and population on foodstamp
double precision :: safetyss, safety(time), safetycntss, safetycnt(time)

!expenditure on Medicaid
double precision,dimension(ng,ne,nh,ns,na,J) :: medicaidexpfunss, medicaidexpfunss1
double precision :: medicaidexpss

!excess demand
double precision :: exdemss, exdem(time)

!fixed endowment cost of being sick (calibrate.f90)
double precision, dimension(nh) :: phi_h

!variables for distributions of assets and labor earnings
double precision, dimension(3,5) :: quintassets, quintlabearnings
double precision, dimension(3,3) :: topassets, toplabearnings
double precision, dimension(3) :: ginicoefassets, ginicoeflabearnings
double precision, dimension(3) :: cvlabearnings, vloglabearnings, avglabearnings, medianlabearnings
double precision, dimension(3) :: cvassets, vlogassets, avgassets, medianassets
!two year persistence for bottom or top given number
double precision, dimension(3) :: pb80, pt99

!variables for variance
double precision, dimension(2) :: coevarl, coevarw

integer :: computeasset

!ACA variables
!pre-existing conditions variables
integer, dimension(time) :: condition
integer :: conditionss

!penalty variables
double precision, dimension(time) :: min_pen, pen_rate, file_min, min_pr
double precision :: min_penss, pen_ratess, file_minss, min_prss

!subsidy variables
integer, dimension(time) :: ACAsub
integer :: ACAsubss

!all buy or not
integer, dimension(time) :: allbuy
integer :: allbuyss


!net spending on ACA subsidies and penalties
double precision,dimension(ng,ne,nh,ns,na,J) :: netfunss, netfunss1

!demographic variables
double precision, dimension(ne,time) :: put_e_t
double precision, dimension(ne,nh,J,time) :: psi_t

!health expenditure shock
double precision, dimension(nh,J,time) :: e_h_t

!fraction on Medicaid by eligibility
double precision :: medicaidtypebyagess(3,J) 


!Warm glow bequests
double precision, parameter :: sigma_bq=2


!Social security
double precision, dimension(time) :: SS_total

!medical expense by age (average and variance)
double precision, dimension(J,ne) :: avg_medical_expense_by_age, var_medical_expense_by_age

!cut off variables
double precision, parameter :: pc1=.6, pc2=.999

contains
! Functions
function U(c,l,f)
implicit none

double precision:: U,c,l,f

	if (c .le. zero) then
	U=umin
	else if (l<zero) then
	U=umin
	else if (l .ge. lendowment) then
	U=umin
	else 
	U=(1.0/(1.0-sigma))*( ( (c**gamma)*((lendowment-f-l)**(1.0-gamma)) )**(1.0-sigma) )
		
! 		if ( l .gt. zero ) then
! 		U=(1.0/(1.0-sigma))*( ( (c**gamma)*((lendowment-phi_h(hc)-phi_l-l)**(1.0-gamma)) )**(1.0-sigma) )
! 		else
! 		U=(1.0/(1.0-sigma))*( ( (c**gamma)*((lendowment-phi_h(hc)-l)**(1.0-gamma)) )**(1.0-sigma) )
! 		end if
! 		
	end if

end function U


subroutine basefun (grid_x,npx,x,vals,inds) 
implicit none

! this subroutine returns the values and the indices of the two basis
! functions that are positive on a given x in the grid_x

double precision,intent(in) :: x
integer , intent(in):: npx
double precision, intent(in) :: grid_x (npx)
double precision, intent(out) ::vals(2)
integer ,intent(out) ::inds(2)
integer :: i
logical :: elem
	
	elem=.false.
	i=1
	do while ( (.not. elem) .and. (i<=npx-1) )
		elem= ((grid_x(i) <= x) .and. (grid_x(i+1) >= x))
		i=i+1 
	end do


	select case(elem)

	case(.true.)
		vals(2)=( x-grid_x(i-1) )/(grid_x(i)-grid_x(i-1))
		vals(1)=( grid_x(i)-x )/(grid_x(i)-grid_x(i-1))
		inds(2)=i
		inds(1)=i-1

	case(.false.)

	if (x>grid_x(npx)) then
	   vals(2)=( x-grid_x(npx-1) )/(grid_x(npx)-grid_x(npx-1))
	   vals(1)=( grid_x(npx)-x )/(grid_x(npx)-grid_x(npx-1))
	   inds(2)=npx
	   inds(1)=npx-1

	endif

    if (x<grid_x(1)) then
	   vals(2)=( x-grid_x(1) )/(grid_x(2)-grid_x(1))
	   vals(1)=( grid_x(2)-x )/(grid_x(2)-grid_x(1))
	   inds(2)=2
	   inds(1)=1
	endif

	end select

end subroutine basefun

end module params